package aspmodificator.parser.gringo;

// Technische Universitaet Wien
// Martin Schwengerer
// C:\Users\Martin\TU Wien\Informatikpraktikum1\Program\cup>java -jar java-cup-11a.jar -interface -symbols Constants -parser Parser -package aspmodificator.parser.gringo ..\Parser.cup


import java_cup.runtime.Symbol;
import java_cup.sym;

%%

%class Lexer
%cup
%implements sym, Constants
%line
%column

%{
 StringBuffer string = new StringBuffer();
int bc = 0;

  private Symbol symbol(int sym) {
    return new Symbol(sym, yyline+1, yycolumn+1);
  }
  
  private Symbol symbol(int sym, Object val) {
    return new Symbol(sym, yyline+1, yycolumn+1, val);
  }
  
  private void error(String message) {
    System.out.println("Error at line "+(yyline+1)+", column "+(yycolumn+1)+" : "+message);
  }
%} 
	NUMBER     = "0"|([1-9][0-9]*)

	ANY        = [\000-\377]

	WS         = [\t\r ]*

	IDENTIFIER = "_"*[a-z][A-Za-z0-9_]*

	LUACALL    = "@"[a-zA-Z_][A-Za-z0-9_]*

	VARIABLE   = "_"*[A-Z][A-Za-z0-9_]*

	ANONYMOUS  = _

	NL = \r|\n|\r\n

%state STRING
%state LINECOMMENT
%state BLOCKCOMMENT



%%


<YYINITIAL> {
"show"           { return symbol(PARSER_SHOW); }
"hide"           { return symbol(PARSER_HIDE); }
"const"          { return symbol(PARSER_CONST); }
"mod"            { return symbol(PARSER_PMOD, yytext());}
"div"            { return symbol(PARSER_PDIV, yytext());}
"pow"            { return symbol(PARSER_PPOW, yytext());}
"abs"            { return symbol(PARSER_PABS, yytext());}
"minimi"[zs]"e"  { return symbol(PARSER_MINIMIZE); }
"maximi"[zs]"e"  { return symbol(PARSER_MAXIMIZE); }
"compute"        { return symbol(PARSER_COMPUTE); }
{WS}			{ /* ignore */ }
{NL}              { /* ignore */ }
"#begin_lua"     	{ /* ignore */ }
"#end_lua" WS "." { /* ignore */ }
"not"            { return symbol(PARSER_NOT); }
"#include"       { return symbol(PARSER_INCLUDE); }
"#show"          { return symbol(PARSER_SHOW); }
"#hide"          { return symbol(PARSER_HIDE); }
"#const"         { return symbol(PARSER_CONST); }
"#minimi"[zs]"e" { return symbol(PARSER_MINIMIZE); }
"#maximi"[zs]"e" { return symbol(PARSER_MAXIMIZE); }
"#compute"       { return symbol(PARSER_COMPUTE); }
"#base"          { return symbol(PARSER_BASE); }
"#cumulative"    { return symbol(PARSER_CUMULATIVE); }
"#volatile"      { return symbol(PARSER_VOLATILE); }
"#domain"        { return symbol(PARSER_DOMAIN); }
"#external"      { return symbol(PARSER_EXTERNAL); }
"#infimum"       { return symbol(PARSER_INFIMUM); }
"#supremum"      { return symbol(PARSER_SUPREMUM); }
{IDENTIFIER}     { return symbol(PARSER_IDENTIFIER, yytext());}
{NUMBER}         { return symbol(PARSER_NUMBER,  new Integer(Integer.parseInt(yytext()))); }
{VARIABLE}       { return symbol(PARSER_VARIABLE, yytext());}
{ANONYMOUS}      { return symbol(PARSER_ANONYMOUS, yytext());}
\"               { string.setLength(0); yybegin(STRING); }
"#sum"           { return symbol(PARSER_SUM, yytext());}
"#count"         { return symbol(PARSER_COUNT, yytext());}
"#min"           { return symbol(PARSER_MIN, yytext());}
"#max"           { return symbol(PARSER_MAX, yytext());}
"#even"          { return symbol(PARSER_EVEN, yytext());}
"#odd"           { return symbol(PARSER_ODD, yytext());}
"#avg"           { return symbol(PARSER_AVG, yytext());}
"#mod"           { return symbol(PARSER_PMOD, yytext());}
"#div"           { return symbol(PARSER_PDIV, yytext());}
"#pow"           { return symbol(PARSER_PPOW, yytext());}
"#abs"           { return symbol(PARSER_PABS, yytext());}
{LUACALL}        { return symbol(PARSER_LUACALL, yytext());}
";;"             { return symbol(PARSER_DSEM); }
";"              { return symbol(PARSER_SEM); }
".."             { return symbol(PARSER_DOTS); }
"."              { return symbol(PARSER_DOT); }
":"              { return symbol(PARSER_COLON); }
":-"             { return symbol(PARSER_IF); }
","              { return symbol(PARSER_COMMA); }
"|"              { return symbol(PARSER_VBAR); }
"["              { return symbol(PARSER_LSBRAC); }
"]"              { return symbol(PARSER_RSBRAC); }
"("              { return symbol(PARSER_LBRAC); }
")"              { return symbol(PARSER_RBRAC); }
"{"              { return symbol(PARSER_LCBRAC); }
"}"              { return symbol(PARSER_RCBRAC); }
"+"              { return symbol(PARSER_PLUS); }
"-"              { return symbol(PARSER_MINUS); }
"**"             { return symbol(PARSER_POW); }
"\\"             { return symbol(PARSER_MOD); }
"*"              { return symbol(PARSER_MULT); }
">"              { return symbol(PARSER_GREATER); }
"<"              { return symbol(PARSER_LOWER); }
">="             { return symbol(PARSER_GTHAN); }
"<="             { return symbol(PARSER_LTHAN); }
"=="             { return symbol(PARSER_EQUAL); }
"!="             { return symbol(PARSER_INEQUAL); }
":="             { return symbol(PARSER_CASSIGN); }
"="              { return symbol(PARSER_ASSIGN); }
"/"              { return symbol(PARSER_SLASH); }
"@"              { return symbol(PARSER_AT); }
"&"              { return symbol(PARSER_AND); }
"^"              { return symbol(PARSER_XOR); }
"~"              { return symbol(PARSER_BNOT); }
"?"              { return symbol(PARSER_QUESTION); }
"%*"             { string.setLength(0); bc++; string.append( "%*" ); yybegin(BLOCKCOMMENT); }
"%"              { string.setLength(0); string.append( "%" ); yybegin(LINECOMMENT); }

/* error fallback */
.|\n              {  error("Illegal character <"+ yytext()+">"); }
}

<STRING> {
  \"                             { yybegin(YYINITIAL);
                                   return symbol(PARSER_STRING, 
                                   string.toString() ); }
  [^\n\r\"\\]+                   { string.append( yytext() ); }
  \\t                            { string.append('\t'); }
  \\n                            { string.append('\n'); }

  \\r                            { string.append('\r'); }
  \\\"                           { string.append('\"'); }
  \\                             { string.append('\\'); }
}

<BLOCKCOMMENT> {
	"*%" 		{ string.append( "*%" );  bc--; if(bc == 0) { yybegin(YYINITIAL); return symbol(PARSER_BLOCKCOMMENT, string.toString()); }  }
	"%*" 		{ bc++; string.append( "%*" );}
	"%"  		{ string.append( "%" );}
	{NL}   	{ string.append("\n"); }
	<<EOF>>	{ yybegin(YYINITIAL); error("BLOCKCOMMENT NOT CLOSED"); }
	{ANY}  	{ string.append(yytext()); }
}

<LINECOMMENT> {
	{NL}   	{ yybegin(YYINITIAL); return symbol(PARSER_LINECOMMENT,string.toString()); }
	<<EOF>>	{ yybegin(YYINITIAL);  if(bc > 0) { error("BLOCKCOMMENT NOT CLOSED"); } else { return symbol(PARSER_LINECOMMENT, string.toString()); } }	
	{ANY}  	{ string.append(yytext()); }
}