/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules;

import java.util.BitSet;
import java.util.EnumMap;
import java.util.HashMap;

import aspmodificator.modificationrules.gringo.ModificationRuleTypeEnum;

public class TabuList {

	private EnumMap<ModificationRuleTypeEnum, HashMap<Integer, BitSet>> tabuList;

	private EnumMap<ModificationRuleTypeEnum, HashMap<Integer, HashMap<Integer, BitSet>>> two_level_tabuList;

	public TabuList() {
		tabuList = new EnumMap<ModificationRuleTypeEnum, HashMap<Integer, BitSet>>(
				ModificationRuleTypeEnum.class);

		two_level_tabuList = new EnumMap<ModificationRuleTypeEnum, HashMap<Integer, HashMap<Integer, BitSet>>>(
				ModificationRuleTypeEnum.class);
	}

	public boolean isTabu(ModificationRuleTypeEnum ruleType, int lineNo,
			int objectIndex) {

		if (!tabuList.containsKey(ruleType))
			return false;

		HashMap<Integer, BitSet> map = tabuList.get(ruleType);

		BitSet bitSet = map.get(Integer.valueOf(lineNo));

		if (bitSet == null)
			return false;

		return bitSet.get(objectIndex);
	}

	public boolean isTabu(ModificationRuleTypeEnum ruleType, int lineNo,
			int objectIndex, int choosenIndex) {

		if (!two_level_tabuList.containsKey(ruleType))
			return false;

		HashMap<Integer, HashMap<Integer, BitSet>> secondLevelMap = two_level_tabuList
				.get(ruleType);

		if (!secondLevelMap.containsKey(lineNo))
			return false;

		HashMap<Integer, BitSet> map = secondLevelMap.get(lineNo);

		BitSet bitSet = map.get(Integer.valueOf(objectIndex));

		if (bitSet == null)
			return false;

		return bitSet.get(choosenIndex);
	}

	public void setTabu(ModificationRuleTypeEnum ruleType, int lineNo,
			int objectIndex) {

		HashMap<Integer, BitSet> map = tabuList.get(ruleType);

		if (map == null) {
			map = new HashMap<Integer, BitSet>();
			tabuList.put(ruleType, map);
		}
		BitSet bitSet = map.get(Integer.valueOf(lineNo));

		if (bitSet == null) {
			bitSet = new BitSet();
			map.put(Integer.valueOf(lineNo), bitSet);
		}

		bitSet.set(objectIndex);
	}

	public void setTabu(ModificationRuleTypeEnum ruleType, int lineNo,
			int objectIndex, int choosenIndex) {
		HashMap<Integer, HashMap<Integer, BitSet>> map = two_level_tabuList
				.get(ruleType);

		if (map == null) {
			map = new HashMap<Integer, HashMap<Integer, BitSet>>();
			two_level_tabuList.put(ruleType, map);
		}

		HashMap<Integer, BitSet> tmpMap = map.get(Integer.valueOf(lineNo));

		if (tmpMap == null) {
			tmpMap = new HashMap<Integer, BitSet>();
			map.put(lineNo, tmpMap);
		}

		BitSet bitSet = tmpMap.get(Integer.valueOf(objectIndex));

		if (bitSet == null) {
			bitSet = new BitSet();
			tmpMap.put(Integer.valueOf(objectIndex), bitSet);
		}

		bitSet.set(choosenIndex);
	}

}
