/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.Literal;

import common.Pair;

public abstract class AbstractComplexLiteralModificationRule extends
		AbstractModificationRule {

	public AbstractComplexLiteralModificationRule() {
		super();
	}

	public AbstractComplexLiteralModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	public int apply(List<LineBase> program) {
		// find applicable rules/objects
		List<Pair<Literal, Literal>> applicableObjectPairs = findApplicableObjects(program);

		if (0 == applicableObjectPairs.size())
			return 0;
		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableObjectPairs.size());

		int applicableObjectNo = applicableObjectPairs.size();

		while (applicableObjectNo > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			applicableObjectNo = applicableObjectPairs.size();

			int objectIndex = -1;
			for (Pair<Literal, Literal> pair : applicableObjectPairs) {

				objectIndex++;

				if (pair.getFirst().isModified()) {
					applicableObjectNo--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(getRuleTypeEnum(), pair.getFirst()
								.getLineNo(), objectIndex)) {
					applicableObjectNo--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {
					Literal newLiteral = modifyLiteral(pair);

					int index = pair.getSecond().getTerms()
							.indexOf(pair.getFirst());
					if (index >= 0)
						pair.getSecond().getTerms().remove(index);

					if (newLiteral != null) {
						pair.getSecond().getTerms().add(index, newLiteral);
						newLiteral.setModified(true);
					}

					ruleApplied++;

					applicableObjectNo--;

					if (useMemory)
						tabuList.setTabu(getRuleTypeEnum(), pair.getFirst()
								.getLineNo(), objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		}

		return ruleApplied;
	}

	protected abstract List<Pair<Literal, Literal>> findApplicableObjects(
			List<LineBase> program);

	/**
	 * Modifies a Literal by replacing it with a new one
	 * 
	 * @param pair
	 *            {@link Pair} with a literal and the containing list
	 * @return the {@link Literal} which should replace the original one. If the
	 *         value is null, no replacement will occur and the literal will
	 *         just BE deleted from the list
	 */
	protected abstract Literal modifyLiteral(Pair<Literal, Literal> pair);

}
