/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
/**
 * 
 */
package aspmodificator.modificationrules.gringo;

import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.Literal;

/**
 * @author Martin Schwengerer
 * 
 */
public abstract class AbstractLiteralModificationRule extends
		AbstractModificationRule {

	public AbstractLiteralModificationRule() {
		super();
	}

	public AbstractLiteralModificationRule(int min, int max) {
		super(min, max);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * aspmodificator.modificationrules.IModificationRule#apply(java.util.List)
	 */
	@Override
	public int apply(List<LineBase> program) {
		// find applicable rules/objects
		List<Literal> applicableObject = findApplicableObjects(program);

		if (0 == applicableObject.size())
			return 0;
		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableObject.size());

		int applicableObjectNo = applicableObject.size();

		while (applicableObjectNo > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			applicableObjectNo = applicableObject.size();
			int objectIndex = -1;

			for (Literal lit : applicableObject) {

				objectIndex++;

				if (lit.isModified()) {
					applicableObjectNo--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(getRuleTypeEnum(), lit.getLineNo(),
								objectIndex)) {
					applicableObjectNo--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {
					modifyLiteral(lit);
					lit.setModified(true);

					ruleApplied++;

					applicableObjectNo--;

					if (useMemory)
						tabuList.setTabu(getRuleTypeEnum(), lit.getLineNo(),
								objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		}

		return ruleApplied;
	}

	protected abstract List<Literal> findApplicableObjects(
			List<LineBase> program);

	protected abstract void modifyLiteral(Literal lit);

}
