/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.Literal;

import common.Tripple;

public abstract class AbstractReplacementLiteralModificationRule<K> extends
		AbstractModificationRule {

	public AbstractReplacementLiteralModificationRule() {
		super();
	}

	public AbstractReplacementLiteralModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	public int apply(List<LineBase> program) {
		// find applicable rules/objects
		List<Tripple<Literal, Literal, List<K>>> applicableObjectPairs = findApplicableObjects(program);

		if (0 == applicableObjectPairs.size())
			return 0;
		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableObjectPairs.size());

		int applicableObjectNo = applicableObjectPairs.size();

		while (applicableObjectNo > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			applicableObjectNo = applicableObjectPairs.size();

			int objectIndex = -1;
			for (Tripple<Literal, Literal, List<K>> tripple : applicableObjectPairs) {

				objectIndex++;

				if (tripple.getFirst().isModified()) {
					applicableObjectNo--;
					continue;
				}

				if (tripple.getThird().isEmpty()) {
					applicableObjectNo--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {

					int choosenIndex = random
							.nextInt(tripple.getThird().size());
					int tries = 0;
					int maxTries = tripple.getThird().size() * 2;
					while (useMemory
							&& tabuList.isTabu(getRuleTypeEnum(), tripple
									.getFirst().getLineNo(), objectIndex,
									choosenIndex) && tries < maxTries) {
						choosenIndex = random
								.nextInt(tripple.getThird().size());
						tries++;
					}

					if (useMemory
							&& tabuList.isTabu(getRuleTypeEnum(), tripple
									.getFirst().getLineNo(), objectIndex,
									choosenIndex)) {
						applicableObjectNo--;
						continue;
					}

					Literal newLiteral = modifyLiteral(tripple, choosenIndex);

					if (newLiteral != tripple.getFirst()) {
						int index = tripple.getSecond().getTerms()
								.indexOf(tripple.getFirst());
						if (index >= 0)
							tripple.getSecond().getTerms().remove(index);

						if (newLiteral != null)
							tripple.getSecond().getTerms()
									.add(index, newLiteral);
					}

					if (newLiteral != null)
						newLiteral.setModified(true);

					ruleApplied++;

					applicableObjectNo--;

					if (useMemory)
						tabuList.setTabu(getRuleTypeEnum(), tripple.getFirst()
								.getLineNo(), objectIndex, choosenIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		}

		return ruleApplied;
	}

	protected abstract List<Tripple<Literal, Literal, List<K>>> findApplicableObjects(
			List<LineBase> program);

	/**
	 * Modifies a Literal by replacing it with a new one
	 * 
	 * @param pair
	 *            {@link Tripple} with a literal, the containing parent object
	 *            and a list of possible substitution values
	 * @return the {@link Literal} which should replace the original one. If the
	 *         value is null, no replacement will occur and the literal will
	 *         just BE deleted from the list
	 */
	protected abstract Literal modifyLiteral(
			Tripple<Literal, Literal, List<K>> tripple, int index);

}
