/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
/**
 * 
 */
package aspmodificator.modificationrules.gringo;

import java.util.List;
import java.util.Random;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;

/**
 * @author Martin
 * 
 */
public abstract class AbstractRuleModificationRule extends
		AbstractModificationRule {

	public AbstractRuleModificationRule() {
		super();
	}

	public AbstractRuleModificationRule(int minApplication, int maxApplication) {
		super(minApplication, maxApplication);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * aspmodificator.modificationrules.IModificationRule#apply(java.util.List)
	 */
	@Override
	public int apply(List<LineBase> program) throws Exception {
		// find applicable rules
		List<GringoRule> applicableRules = findApplicableRules(program);

		if (applicableRules.size() == 0)
			return 0;

		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableRules.size());

		int applicableObjectNo = applicableRules.size();

		while (applicableObjectNo > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			applicableObjectNo = applicableRules.size();

			int objectIndex = -1;
			for (GringoRule rule : applicableRules) {

				objectIndex++;

				if (rule.isModified()) {
					applicableObjectNo--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(getRuleTypeEnum(), 1, objectIndex)) {
					applicableObjectNo--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {
					// modify the rule
					modifyRule(rule);

					rule.setModified(true);

					ruleApplied++;

					applicableObjectNo--;

					if (useMemory)
						tabuList.setTabu(getRuleTypeEnum(), 1, objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		}

		return ruleApplied;
	}

	protected abstract List<GringoRule> findApplicableRules(
			List<LineBase> program);

	protected abstract boolean modifyRule(GringoRule rule);

	@Override
	public void setRandom(Random rand) {
		this.random = rand;
	}

}
