/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Predicate;

public class AddStrongNegationModificationRule extends
		SwapStrongNegationModificationRule {
	public AddStrongNegationModificationRule() {
		super();
	}

	public AddStrongNegationModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	public AddStrongNegationModificationRule(int minApplication,
			int maxApplication, boolean requireExistingPredicate) {
		super(minApplication, maxApplication, requireExistingPredicate);
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		List<Literal> negatedLiterals = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					for (Literal lit : rule.getHead()) {
						if (Predicate.class.isAssignableFrom(lit.getClass())
								&& !lit.isModified()) {
							Predicate pred = (Predicate) lit;
							if (!pred.isNegated())
								result.add(lit);
							else if (requireExistingObject)
								negatedLiterals.add(lit);
						}
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					for (Literal lit : rule.getBody()) {
						if (Predicate.class.isAssignableFrom(lit.getClass())
								&& !lit.isModified()) {
							Predicate pred = (Predicate) lit;
							if (!pred.isNegated())
								result.add(lit);
							else if (requireExistingObject)
								negatedLiterals.add(lit);
						}
					}
				}
			}
		}

		if (requireExistingObject) {
			// remove literals where the un-negated literal does not occur in
			// the program

			ArrayList<Literal> toRemove = new ArrayList<Literal>();

			for (Literal lit : result) {
				boolean found = false;

				for (Literal unNegLit : negatedLiterals) {
					if (!unNegLit.getIdentifier().equals(lit.getIdentifier()))
						continue;

					if (unNegLit.getArity() != lit.getArity())
						continue;

					found = true;
				}

				if (!found)
					toRemove.add(lit);

			}

			result.removeAll(toRemove);
		}

		return result;
	}

	@Override
	protected void modifyLiteral(Literal lit) {

		Predicate pred = (Predicate) lit;
		pred.setNegated(true);

		if (writeComments) {
			lit.addSucceedingComment(COMMENT_ADDED);
		}
	}

}
