/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.CommentLine;
import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;

public class DeleteRuleModificationRule extends AbstractModificationRule {

	private LogicRuleTypeEnum ruleType = LogicRuleTypeEnum.ALL;

	public DeleteRuleModificationRule() {
		super();
	}

	public DeleteRuleModificationRule(int min, int max) {
		super(min, max);
	}

	@Override
	public int apply(List<LineBase> program) {

		// find applicable rules
		List<GringoRule> applicableRules = findApplicableRules(program);

		/*
		 * if (applicableRules.size() == 0) { throw new
		 * NoApplicablerulesFoundException( "No applicable rules for " +
		 * this.toString() + " found."); }
		 */
		if (applicableRules.size() == 0)
			return 0;

		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableRules.size());

		int noApplicapleObject = 0;

		do {
			noApplicapleObject = applicableRules.size();

			int objectIndex = -1;
			for (GringoRule rule : applicableRules) {

				objectIndex++;

				if (rule.isModified()) {
					noApplicapleObject--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(ModificationRuleTypeEnum.RDP, 1,
								objectIndex)) {
					noApplicapleObject--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {

					int index = program.indexOf(rule);

					program.remove(index);

					if (writeComments) {
						program.add(
								index,
								new CommentLine(
										"%* ModificationNote: The following rule has been removed: "
												+ rule.toOutputString() + " *%",
										rule.getStartLineNo()));
					}

					rule.setModified(true);

					ruleApplied++;

					noApplicapleObject--;

					if (useMemory)
						tabuList.setTabu(ModificationRuleTypeEnum.RDP, 1,
								objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		} while (noApplicapleObject > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication);

		/*
		 * if (ruleApplied < minApplication) throw new
		 * NoApplicablerulesFoundException( "Only " + ruleApplied +
		 * " rules to modify found. Minimum application for this rule was: " +
		 * minApplication);
		 */
		return ruleApplied;
	}

	private List<GringoRule> findApplicableRules(List<LineBase> program) {
		List<GringoRule> result = new ArrayList<GringoRule>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				switch (ruleType) {
				case FACT:// look only for facts
					if (rule.getHead() != null
							&& !rule.getHead().isEmpty()
							&& (rule.getBody() == null || rule.getBody()
									.isEmpty()))
						result.add(rule);
					break;
				case CONSTRAINT: // look only for constraints
					if ((rule.getHead() == null || rule.getHead().isEmpty())
							&& rule.getBody() != null
							&& !rule.getBody().isEmpty())
						result.add(rule);
					break;
				case PROPER: // look only for rules including head and body
					if (rule.getHead() != null && !rule.getHead().isEmpty()
							&& rule.getBody() != null
							&& !rule.getBody().isEmpty())
						result.add(rule);
					break;
				case ALL:// include all rules
				default:
					result.add(rule);
					break;
				}
			}
		}

		return result;
	}

	public LogicRuleTypeEnum getRuleType() {
		return ruleType;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.RDP;
	}

	public void setRuleType(LogicRuleTypeEnum ruleType) {
		this.ruleType = ruleType;
	}

}
