/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.AggregateLiteral;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.ICommentable;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.PoolTerm;

import common.Tripple;

public class DeleteTermModificationRule extends AbstractModificationRule {
	protected boolean includeInsideLiterals = true;

	public DeleteTermModificationRule() {
	}

	public DeleteTermModificationRule(int minApplication, int maxApplication) {
		super(minApplication, maxApplication);
	}

	private void addInsideLiterals(
			List<Tripple<Literal, List<Literal>, ICommentable>> result,
			GringoRule rule, Literal lit) {
		boolean safe;
		// look for literals inside aggregates, Pool terms, etc

		if (containsRemovableLiterals(lit)) {
			if (lit.getTerms() != null && lit.getTerms().size() > 1) {

				for (int j = 0; j < lit.getTerms().size(); j++) {
					Literal lit2 = lit.getTerms().remove(j);
					safe = GringoRuleUtil.checkSafety(rule);
					if (safe) {
						ICommentable commentable = null;

						if (j < (lit.getTerms().size())) {
							commentable = lit.getTerms().get(j);
						} else {
							commentable = lit.getTerms().get(j - 1);
						}

						result.add(new Tripple<Literal, List<Literal>, ICommentable>(
								lit2, lit.getTerms(), commentable));
					}

					lit.getTerms().add(j, lit2);
				}
			}
		}
	}

	@Override
	public int apply(List<LineBase> program) {
		// find applicable rules/objects
		List<Tripple<Literal, List<Literal>, ICommentable>> applicableObjectPairs = findApplicableObjects(program);

		/*
		 * if (0 == applicableObject.size()) { throw new
		 * NoApplicablerulesFoundException( "No applicable objects for " +
		 * this.toString() + " found."); }
		 */

		if (0 == applicableObjectPairs.size())
			return 0;
		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableObjectPairs.size());

		int noApplicableObjects = applicableObjectPairs.size();

		while (noApplicableObjects > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			noApplicableObjects = applicableObjectPairs.size();

			int objectIndex = -1;
			for (Tripple<Literal, List<Literal>, ICommentable> tripple : applicableObjectPairs) {

				objectIndex++;

				if (tripple.getFirst().isModified()) {
					noApplicableObjects--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(getRuleTypeEnum(), tripple
								.getFirst().getLineNo(), objectIndex)) {
					noApplicableObjects--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {
					Literal newLiteral = modifyLiteral(tripple);

					int index = tripple.getSecond().indexOf(tripple.getFirst());
					if (index >= 0)
						tripple.getSecond().remove(index);

					if (newLiteral != null) {
						tripple.getSecond().add(index, newLiteral);
						newLiteral.setModified(true);
					}

					ruleApplied++;

					noApplicableObjects--;

					if (useMemory)
						tabuList.setTabu(getRuleTypeEnum(), tripple.getFirst()
								.getLineNo(), objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}

			// applicableObjectPairs.removeAll(toRemove);
		}

		/*
		 * if (ruleApplied < minApplication) throw new
		 * NoApplicablerulesFoundException( "Only " + ruleApplied +
		 * " objects to modify found. Minimum application for this rule was: " +
		 * minApplication);
		 */
		return ruleApplied;
	}

	private boolean containsRemovableLiterals(Literal lit) {
		if (AggregateLiteral.class.isAssignableFrom(lit.getClass()))
			return true;
		if (PoolTerm.class.isAssignableFrom(lit.getClass()))
			return true;
		return false;
	}

	protected List<Tripple<Literal, List<Literal>, ICommentable>> findApplicableObjects(
			List<LineBase> program) {
		List<Tripple<Literal, List<Literal>, ICommentable>> result = new ArrayList<Tripple<Literal, List<Literal>, ICommentable>>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& rule.getHead().size() > 0
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					int headSize = rule.getHead().size();
					for (int i = 0; i < headSize; i++) {
						if (rule.getBody() != null && !rule.getBody().isEmpty()) {

							Literal lit = rule.getHead().remove(i);
							boolean safe = GringoRuleUtil.checkSafety(rule);
							if (safe) {
								ICommentable commentable = null;
								if (i == (headSize - 1))
									commentable = rule.getCenterCommentable();
								else {
									if (i < (rule.getHead().size() - 1)) {
										commentable = rule.getHead().get(i + 1);
									} else {
										commentable = rule
												.getCenterCommentable();
									}
								}

								result.add(new Tripple<Literal, List<Literal>, ICommentable>(
										lit, rule.getHead(), commentable));
							}

							rule.getHead().add(i, lit);
						}

						if (includeInsideLiterals) {
							addInsideLiterals(result, rule,
									rule.getHead().get(i));
						}
					}

				}
				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.BODY || mrRuleTarget == RuleTargetEnum.ALL)) {
					int bodySize = rule.getBody().size();
					for (int i = 0; i < bodySize; i++) {
						if (rule.getBody().size() > 1
								|| (rule.getHead() != null && !rule.getHead()
										.isEmpty())) {
							Literal lit = rule.getBody().remove(i);
							boolean safe = GringoRuleUtil.checkSafety(rule);

							if (safe) {
								ICommentable commentable = null;
								if (i == (bodySize - 1))
									commentable = rule.getBackCommentable();
								else {
									if (i < (rule.getBody().size())) {
										commentable = rule.getBody().get(i);
									} else {
										commentable = rule.getBackCommentable();
									}
								}
								result.add(new Tripple<Literal, List<Literal>, ICommentable>(
										lit, rule.getBody(), commentable));
							}

							rule.getBody().add(i, lit);
						}

						if (includeInsideLiterals) {
							addInsideLiterals(result, rule,
									rule.getBody().get(i));
						}
					}
				}
			}
		}
		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		switch (mrRuleTarget) {
		case HEAD:
			return ModificationRuleTypeEnum.LDH;
		case BODY:
			return ModificationRuleTypeEnum.LDB;
		case ALL:
		default:
			return ModificationRuleTypeEnum.LDA;
		}
	}

	public boolean isIncludeInsideLiterals() {
		return includeInsideLiterals;
	}

	/**
	 * Modifies a Literal by replacing it with a new one
	 * 
	 * @param tripple
	 *            {@link Tripple} with a literal, the containing list and the
	 *            object where the comments should be added (if any)
	 * @return the {@link Literal} which should replace the original one. If the
	 *         value is null, no replacement will occur and the literal will
	 *         just be deleted from the list
	 */
	protected Literal modifyLiteral(
			Tripple<Literal, List<Literal>, ICommentable> tripple) {
		if (writeComments) {

			ICommentable commentable = tripple.getThird();
			if (commentable != null)
				commentable
						.addPrecedingComment("%*ModificationNote: Literal "
								+ tripple.getFirst().toString()
								+ " has been removed*%");
		}
		return null;
	}

	public void setIncludeInsideLiterals(boolean includeInsideLiterals) {
		this.includeInsideLiterals = includeInsideLiterals;
	}

}
