/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.ConstTerm;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.MathTerm;
import aspmodificator.parser.gringo.RangeTerm;
import aspmodificator.parser.gringo.Term;

public class RangeTermModificationRule extends AbstractLiteralModificationRule {

	public RangeTermModificationRule() {
		super();
	}

	public RangeTermModificationRule(int minApplication, int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					for (Literal lit : rule.getHead()) {
						if (lit.isModified())
							continue;

						List<Literal> tmpList = Literal.findClassesByType(
								RangeTerm.class, lit);

						for (Literal mt : tmpList) {
							if (!mt.isModified())
								result.add(mt);
						}
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					for (Literal lit : rule.getBody()) {
						if (lit.isModified())
							continue;

						List<Literal> tmpList = Literal.findClassesByType(
								RangeTerm.class, lit);

						for (Literal mt : tmpList) {
							if (!mt.isModified())
								result.add(mt);
						}
					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TODO;
	}

	@Override
	protected void modifyLiteral(Literal lit) {

		boolean modifyLower = random.nextBoolean();
		boolean shouldIncrement = random.nextBoolean();

		RangeTerm rangeTerm = (RangeTerm) lit;
		Integer lower = null;
		Integer upper = null;

		if (ConstTerm.class.isAssignableFrom(rangeTerm.getLowerRange()
				.getClass())) {
			lower = Integer.valueOf(rangeTerm.getLowerRange().getIdentifier());
		}

		if (ConstTerm.class.isAssignableFrom(rangeTerm.getUpperRange()
				.getClass())) {
			upper = Integer.valueOf(rangeTerm.getUpperRange().getIdentifier());
		}

		boolean onlyOneDifferent = false;
		if (lower != null && upper != null && (lower + 1) == upper) {
			onlyOneDifferent = true;
		}

		if (modifyLower) {
			Literal lowerRange = rangeTerm.getLowerRange();

			if (lower != null) {
				if (onlyOneDifferent || !shouldIncrement) {
					lowerRange
							.setIdentifier(Integer.toString(lower.intValue() - 1));
					if (writeComments) {
						lowerRange
								.addSucceedingComment("%* ModificationNote: lower value has been decreased. Original value was "
										+ lower + "  *%");
					}
				} else {
					lowerRange
							.setIdentifier(Integer.toString(lower.intValue() + 1));
					if (writeComments) {
						lowerRange
								.addSucceedingComment("%* ModificationNote: lower value has been increased. Original value was "
										+ lower + "  *%");
					}
				}
			} else {
				if (onlyOneDifferent || !shouldIncrement) {
					MathTerm mathTerm = new MathTerm(MathTerm.MINUS,
							(Term) lowerRange,
							new ConstTerm(Integer.valueOf(1), lowerRange
									.getLineNo()), lowerRange.getLineNo());

					rangeTerm.setLowerRange(mathTerm);
					if (writeComments) {
						rangeTerm
								.addSucceedingComment("%* ModificationNote: lower value has been decreased. Original value was "
										+ lower + "  *%");
					}
				} else {
					MathTerm mathTerm = new MathTerm(MathTerm.PLUS,
							(Term) lowerRange,
							new ConstTerm(Integer.valueOf(1), lowerRange
									.getLineNo()), lowerRange.getLineNo());

					rangeTerm.setLowerRange(mathTerm);
					if (writeComments) {
						rangeTerm
								.addSucceedingComment("%* ModificationNote: lower value has been increased. Original value was "
										+ lower + "  *%");
					}
				}
			}
		} else {

			Literal upperRange = rangeTerm.getUpperRange();

			if (upper != null) {
				if (onlyOneDifferent || shouldIncrement) {
					upperRange
							.setIdentifier(Integer.toString(upper.intValue() + 1));
					if (writeComments) {
						upperRange
								.addSucceedingComment("%* ModificationNote: upper value has been increased. Original value was "
										+ upper + "  *%");
					}
				} else {
					upperRange
							.setIdentifier(Integer.toString(upper.intValue() - 1));
					if (writeComments) {
						upperRange
								.addSucceedingComment("%* ModificationNote: upper value has been decreased. Original value was "
										+ upper + "  *%");
					}
				}
			} else {
				if (onlyOneDifferent || shouldIncrement) {
					MathTerm mathTerm = new MathTerm(MathTerm.PLUS,
							(Term) upperRange,
							new ConstTerm(Integer.valueOf(1), upperRange
									.getLineNo()), upperRange.getLineNo());

					rangeTerm.setUpperRange(mathTerm);
					if (writeComments) {
						rangeTerm
								.addSucceedingComment("%* ModificationNote: upper value has been increased. Original value was "
										+ upper + "  *%");
					}
				} else {
					MathTerm mathTerm = new MathTerm(MathTerm.MINUS,
							(Term) upperRange,
							new ConstTerm(Integer.valueOf(1), upperRange
									.getLineNo()), upperRange.getLineNo());

					rangeTerm.setUpperRange(mathTerm);
					if (writeComments) {
						rangeTerm
								.addSucceedingComment("%* ModificationNote: lower value has been decreased. Original value was "
										+ upper + "  *%");
					}
				}
			}
			/*
			 * if (onlyOneDifferent || shouldIncrement) {
			 * upperConstant.setIdentifier(Integer.toString(upper + 1)); if
			 * (writeComments) { lowerConstant .addSucceedingComment(
			 * "%* ModificationNote: upper value has been decreased. Original value was "
			 * + upper + "  *%"); } } else {
			 * upperConstant.setIdentifier(Integer.toString(upper - 1)); if
			 * (writeComments) { lowerConstant .addSucceedingComment(
			 * "%* ModificationNote: upper value has been decreased. Original value was "
			 * + upper + "  *%"); } }
			 */
		}
	}
}
