/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.RelLit;

public class RelationOperatorModificationRule extends
		AbstractLiteralModificationRule {

	protected static final String COMMENT_RELATION_OPERATOR_MODIFICATION = "%*ModificationNote: relation-operator has been modified. Original operator was ";

	public static final String[] operators = { RelLit.EQUAL, RelLit.LTHAN,
			RelLit.GTHAN, RelLit.LOWER, RelLit.GREATER, RelLit.INEQUAL };

	public static final String[] limitedOperatorList = { RelLit.EQUAL,
			RelLit.LTHAN, RelLit.GTHAN };

	public boolean avoidNearlyInequalOperators = false;

	public RelationOperatorModificationRule() {
		super();
	}

	public RelationOperatorModificationRule(int minApplication,
			int maxApplication, boolean avoidNearlyInequalOperators) {
		super(minApplication, maxApplication);
		this.avoidNearlyInequalOperators = avoidNearlyInequalOperators;
	}

	private boolean checkPreconditions(Literal lit) {
		if (RelLit.class.isAssignableFrom(lit.getClass()))
			return true;
		return false;
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					for (Literal lit : rule.getHead()) {
						if (checkPreconditions(lit) && !lit.isModified())
							result.add(lit);
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					for (Literal lit : rule.getBody()) {
						if (checkPreconditions(lit) && !lit.isModified())
							result.add(lit);
					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.LRC;
	}

	public boolean isAvoidNearlyInequalOperators() {
		return avoidNearlyInequalOperators;
	}

	@Override
	protected void modifyLiteral(Literal lit) {
		RelLit relLit = (RelLit) lit;

		if (writeComments) {
			relLit.addSucceedingComment(COMMENT_RELATION_OPERATOR_MODIFICATION
					+ lit.getIdentifier() + " *%");
		}

		int index = random.nextInt(operators.length);

		if (avoidNearlyInequalOperators) {
			// if the literal was !=, > or <, it should not be changed to one of
			// these literals
			if (lit.getIdentifier().equals(RelLit.LOWER)
					|| lit.getIdentifier().equals(RelLit.GREATER)
					|| lit.getIdentifier().equals(RelLit.INEQUAL)) {
				index = random.nextInt(limitedOperatorList.length);
			}
		} else {
			while (operators[index].equals(lit.getIdentifier()))
				index = random.nextInt(operators.length);
		}

		relLit.setIdentifier(operators[index]);
	}

	public void setAvoidNearlyInequalOperators(
			boolean avoidNearlyInequalOperators) {
		this.avoidNearlyInequalOperators = avoidNearlyInequalOperators;
	}

}
