/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.AggregateLiteral;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.RelLit;

public class SwapTermModificationRule extends AbstractLiteralModificationRule {

	public SwapTermModificationRule() {
		super();
	}

	public SwapTermModificationRule(int minApplication, int maxApplication) {
		super(minApplication, maxApplication);
	}

	private boolean checkPreconditions(Literal lit) {

		if (RelLit.class.isAssignableFrom(lit.getClass())
				&& ((RelLit) lit).getIdentifier().equals(RelLit.ASSIGN))
			return false;

		if (AggregateLiteral.class.equals(lit.getClass()))
			return false;

		if (lit != null && lit.getTerms() != null && lit.getTerms().size() > 1) {
			// check if the literals are equal - if so, an exchange would not
			// change anything
			for (Literal l1 : lit.getTerms()) {
				boolean foundNoOther = true;
				for (Literal l2 : lit.getTerms()) {
					if (l1 == l2)
						continue;

					if (!l1.toString().equals(l2.toString())) {
						foundNoOther = false;
						break;
					}
				}

				if (foundNoOther)
					return false;
			}
			return true;
		} else
			return false;
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					for (Literal lit : rule.getHead()) {
						if (checkPreconditions(lit) && !lit.isModified())
							result.add(lit);
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					for (Literal lit : rule.getBody()) {
						if (checkPreconditions(lit) && !lit.isModified())
							result.add(lit);
					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TST;
	}

	@Override
	protected void modifyLiteral(Literal lit) {
		if (writeComments) {
			String original = lit.toString();
			lit.addSucceedingComment("%* ModificationNote: Terms have been swapped. Original literal: "
					+ original + "*%");
		}

		if (lit.getTerms().size() > 2) {
			int index1 = random.nextInt(lit.getTerms().size());
			int index2 = random.nextInt(lit.getTerms().size());

			Literal l1 = lit.getTerms().get(index1);
			Literal l2 = lit.getTerms().get(index2);

			while (index1 == index2 && l1.toString().equals(l2.toString())) {
				index2 = random.nextInt(lit.getTerms().size());
				l1 = lit.getTerms().get(index1);
				l2 = lit.getTerms().get(index2);
			}

			lit.getTerms().remove(index1);
			lit.getTerms().add(index1, l2);

			lit.getTerms().remove(index2);
			lit.getTerms().add(index2, l1);
		} else {
			Literal tmp = lit.getTerms().remove(0);
			lit.getTerms().add(tmp);
		}
	}

}
