/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.RelLit;
import aspmodificator.parser.gringo.Term;

public class SwitchDefaultNegationModificationRule extends
		AbstractLiteralModificationRule {

	public static final String COMMENT_ADDED = "%*ModificationNote: added default-negation \"not\" *%";
	public static final String COMMENT_REMOVED = "%*ModificationNote: removed default-negation \"not\" *%";

	public SwitchDefaultNegationModificationRule() {
		super();
	}

	public SwitchDefaultNegationModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	protected boolean allowsDefaultNegation(Class<? extends Literal> clazz) {

		if (Term.class.isAssignableFrom(clazz))
			return false;

		if (RelLit.class.isAssignableFrom(clazz))
			return false;

		return true;
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getBody() != null) {
					for (Literal lit : rule.getBody()) {
						if (allowsDefaultNegation(lit.getClass())
								&& !lit.isModified()) {

							if (!lit.isSigned()) {
								// test if resulting rule is safe
								lit.setSigned(true);

								if (GringoRuleUtil.checkSafety(rule))
									result.add(lit);

								lit.setSigned(false);
							} else {
								result.add(lit);
							}
						}
					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.SDN;
	}

	@Override
	protected void modifyLiteral(Literal lit) {
		lit.setSigned(!lit.isSigned());

		if (writeComments) {
			if (lit.isSigned())
				lit.addSucceedingComment(COMMENT_ADDED);
			else
				lit.addSucceedingComment(COMMENT_REMOVED);
		}
	}

}
