/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.List;

public class CondLit extends Literal {

	private Literal literal;
	private Term weight;
	private boolean defaultWeight = true;

	protected CondLit() {
		super();
	}

	public CondLit(Literal lit, Term weight, List<Literal> cond, int lineNo,
			boolean isDefaultWeight) {
		super(null, cond, lineNo);
		this.literal = lit;
		this.weight = weight;
		this.defaultWeight = isDefaultWeight;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		CondLit condLit = (CondLit) super.clone();

		Literal newLit = null;
		if (literal != null)
			newLit = (Literal) literal.clone();

		Term newWeight = null;
		if (weight != null)
			newWeight = (Term) weight.clone();
		condLit.setLiteral(newLit);
		condLit.setWeight(newWeight);
		return condLit;
	}

	public Term getWeight() {
		return weight;
	}

	public boolean hasDefaultWeight() {
		return defaultWeight;
	}

	private void setLiteral(Literal lit) {
		this.literal = lit;
	}

	public void setWeight(Term weight) {
		this.weight = weight;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();

		if (precedingComments != null && precedingComments.size() > 0) {
			for (BlockComment bc : precedingComments) {
				sb.append(bc.getText());
			}
		}

		if (literal != null) {
			sb.append(literal.toString());
		}

		if (terms != null && terms.size() > 0) {
			sb.append(" : ");
			for (int i = 0; i < terms.size() - 1; i++) {
				sb.append(terms.get(i).toString());
				sb.append(", ");
			}
			sb.append(terms.get(terms.size() - 1).toString());
		}

		if (weight != null) {
			if (!weight.toString().equals("1") || !hasDefaultWeight()) {
				sb.append(" = ");
				sb.append(weight.toString());
			}
		}

		if (succeedingComments != null && succeedingComments.size() > 0) {
			for (BlockComment bc : succeedingComments) {
				sb.append(bc.getText());
			}
		}

		return sb.toString();
	}

}
