/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;

public class DomainDefinition extends LineBase {

	private Signed identifier;
	private List<VarTerm> vars;

	public DomainDefinition(Signed id, List<VarTerm> vars) {
		super();
		this.identifier = id;
		this.vars = vars;
	}

	public DomainDefinition(Signed id, List<VarTerm> vars, int lineNo) {
		super(lineNo);
		this.identifier = id;
		this.vars = vars;

	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		List<VarTerm> newVars = null;
		if (vars != null) {
			newVars = new ArrayList<VarTerm>();
			for (VarTerm t : vars) {
				newVars.add((VarTerm) t.clone());
			}
		}

		return new DomainDefinition((Signed) identifier.clone(), newVars,
				startLineNo);
	}

	@Override
	public String toOutputString() {
		return toString();
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("#domain");
		if (identifier != null)
			sb.append(identifier.toString());

		if (vars != null && vars.size() > 0) {
			sb.append("(");
			for (int i = 0; i < vars.size() - 1; i++) {
				sb.append(vars.get(i).toString());
				sb.append(", ");
			}
			sb.append(vars.get(vars.size() - 1).toString());
			sb.append(")");
		}
		return sb.toString();
	}

}
