/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;

public class ExternalStatement extends LineBase {
	private Predicate pred;
	private List<Literal> list;
	private Integer num;

	public ExternalStatement(Predicate pred, List<Literal> list, int startLineNo) {
		super(startLineNo);
		this.pred = pred;
		this.list = list;
	}

	public ExternalStatement(Signed id, Integer num, int startLineNo) {
		super(startLineNo);
		this.pred = id;
		this.num = num;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		if (list != null) {
			List<Literal> newList = new ArrayList<Literal>();
			for (Literal lit : list) {
				newList.add((Literal) lit.clone());
			}
			return new ExternalStatement((Predicate) pred.clone(), newList,
					startLineNo);
		} else if (num != null) {
			return new ExternalStatement((Signed) pred.clone(),
					new Integer(num), startLineNo);
		} else {
			return new ExternalStatement((Predicate) pred.clone(), null,
					startLineNo);
		}
	}

	@Override
	public String toOutputString() {
		StringBuffer sb = new StringBuffer();

		sb.append("#external ");
		sb.append(pred.toString());

		if (list != null) {
			sb.append(" ");

			Literal lit;
			int currentLineNo = 0;

			// initialize currentLineNo
			currentLineNo = list.get(0).getLineNo();

			for (int i = 0; i < (list.size() - 1); i++) {

				lit = list.get(i);
				while (lit.getLineNo() > currentLineNo) {
					sb.append("\n");
					currentLineNo++;
				}

				sb.append(lit.toString());
				sb.append(" : ");
			}

			lit = list.get(list.size() - 1);
			while (lit.getLineNo() > currentLineNo) {
				sb.append("\n");
				currentLineNo++;
			}
			sb.append(lit.toString());

		} else if (num != null) {
			sb.append("/ ");
			sb.append(num.toString());
		}
		sb.append(".");
		return sb.toString();
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();

		sb.append("#external ");
		sb.append(pred.toString());

		if (list != null) {
			sb.append(" ");
			Literal lit;

			for (int i = 0; i < (list.size() - 1); i++) {
				lit = list.get(i);

				sb.append(lit.toString());
				sb.append(" : ");
			}

			lit = list.get(list.size() - 1);

			sb.append(lit.toString());

		} else if (num != null) {
			sb.append("/ ");
			sb.append(num.toString());
		}
		sb.append(".");
		return sb.toString();
	}

}
