/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class MathTerm extends Term {

	public static final String DIV = "/";
	public static final String MOD = "\\";
	public static final String PLUS = "+";
	public static final String MINUS = "-";
	public static final String POW = "**";
	public static final String MULT = "*";

	public static final String ABS = "#abs";

	public static final String AND = "&";
	public static final String XOR = "^";
	public static final String BNOT = "~";

	public static final String SUPREMUM = "SUPREMUM";
	public static final String INFIMUM = "INFIMUM";

	public static final String OR = null; // not used in gringo???

	// private Term firstTerm;
	// private Term secondTerm;

	public static Term CreateNewMinusOne() {
		return new ConstTerm(-1);
	}

	public static Term CreateNewOne() {
		return new ConstTerm(1);
	}

	public static Term CreateNewZero() {
		return new ConstTerm(0);
	}

	protected MathTerm() {
		super();
	}

	public MathTerm(String operant, Literal a) {
		super(operant);
		List<Literal> tmpList = new ArrayList<Literal>();
		tmpList.add(a);
		this.setTerms(tmpList);
		// this.firstTerm = a;
		// this.secondTerm = null;
	}

	public MathTerm(String operant, Literal a, int lineNo) {
		super(operant, lineNo);
		List<Literal> tmpList = new ArrayList<Literal>();
		tmpList.add(a);
		this.setTerms(tmpList);
	}

	public MathTerm(String operant, Literal a, Literal b) {
		super(operant);
		List<Literal> tmpList = new ArrayList<Literal>();
		tmpList.add(a);
		tmpList.add(b);
		this.setTerms(tmpList);
		// this.firstTerm = a;
		// this.secondTerm = b;
	}

	public MathTerm(String operant, Literal a, Literal b, int lineNo) {
		super(operant, lineNo);
		List<Literal> tmpList = new ArrayList<Literal>();
		tmpList.add(a);
		tmpList.add(b);
		this.setTerms(tmpList);
	}

	private boolean hashHigherBinding(String identifier, Literal literal) {
		if (literal == null)
			return false;

		if (literal.getTerms() == null || literal.getTerms().size() <= 1)
			return false;

		if (MathTerm.class.isAssignableFrom(literal.getClass()) == false)
			return false;

		MathTerm m2 = (MathTerm) literal;

		if ((identifier.equals(MULT) || identifier.equals(DIV))
				&& (m2.identifier.equals(PLUS) || m2.identifier.equals(MINUS)))
			return true;

		if ((identifier.equals(POW))
				&& (m2.identifier.equals(PLUS) || m2.identifier.equals(MINUS)
						|| identifier.equals(MULT) || identifier.equals(DIV)))
			return true;

		return false;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (precedingComments != null && precedingComments.size() > 0) {
			for (BlockComment bc : precedingComments) {
				sb.append(bc.getText());
			}
		}

		if (identifier.equals(MINUS) && terms != null && terms.size() == 2
				&& terms.get(0).toString().equals("0")) {
			// special case for "0-3" => -3
			sb.append(identifier);
			sb.append(terms.get(1).toString());
		} else if (terms != null && terms.size() == 2) {
			if (hashHigherBinding(identifier, terms.get(0))) {
				sb.append("(");
				sb.append(terms.get(0).toString());
				sb.append(")");
			} else {
				sb.append(terms.get(0).toString());
			}

			sb.append(identifier);

			if (hashHigherBinding(identifier, terms.get(1))) {
				sb.append("(");
				sb.append(terms.get(1).toString());
				sb.append(")");
			} else {
				sb.append(terms.get(1).toString());
			}
		} else if (terms != null && terms.size() == 1) {
			sb.append(identifier);
			sb.append("(");
			sb.append(terms.get(0).toString());
			sb.append(")");
		} else {
			sb.append(identifier);
		}

		if (succeedingComments != null && succeedingComments.size() > 0) {
			for (BlockComment bc : succeedingComments) {
				sb.append(bc.getText());
			}
		}
		return sb.toString();
	}
}
