/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class OptimizeLiteral {

	private Literal head;
	private Term weight;
	private Term prio;
	private List<Literal> body;
	private int lineNo = -1;

	public OptimizeLiteral(Literal head, Term weight, Term prio,
			List<Literal> body) {
		this.head = head;
		this.weight = weight;
		this.prio = prio;
		this.body = body;
	}

	public OptimizeLiteral(Literal head, Term weight, Term prio,
			List<Literal> body, int lineNo) {
		this.head = head;
		this.weight = weight;
		this.prio = prio;
		this.body = body;
		this.lineNo = lineNo;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		Literal newHead = null;

		if (head != null)
			newHead = (Literal) head.clone();

		Term newWeight = null;

		if (weight != null)
			newWeight = (Term) weight.clone();

		Term newPrio = null;
		if (prio != null)
			newPrio = (Term) prio.clone();

		List<Literal> newBody = new ArrayList<Literal>();

		if (body != null && body.size() > 0) {
			for (Literal lit : body) {
				newBody.add((Literal) lit.clone());
			}
		}
		return new OptimizeLiteral(newHead, newWeight, newPrio, newBody, lineNo);
	}

	public int getLineNo() {
		return lineNo;
	}

	@Override
	public String toString() {
		// minimize [ hotel(X) : cost(X,Y) : star(X,Z) = Y/Z ].
		StringBuffer sb = new StringBuffer();
		if (head != null) {
			sb.append(head.toString());
		}

		if (body != null) {
			sb.append(" : ");
			for (int i = 0; i < body.size() - 1; i++) {
				sb.append(body.get(i).toString());
				sb.append(" : ");
			}

			sb.append(body.get(body.size() - 1).toString());
		}

		if (weight != null) {
			sb.append(" = ");
			sb.append(weight.toString());
		}

		if (prio != null) {
			sb.append(" @");
			sb.append(prio.toString());
		}

		return sb.toString();
	}
}
