/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class Predicate extends Literal {

	private boolean negated;

	protected Predicate() {
		super();
	}

	public Predicate(String id, List<Term> terms, boolean negated) {
		super(id, null, false);
		if (terms != null) {
			this.setTerms(new ArrayList<Literal>(terms));
		}
		this.negated = negated;
	}

	public Predicate(String id, List<Term> terms, boolean negated, int lineNo) {
		super(id, null, false, lineNo);
		if (terms != null) {
			this.setTerms(new ArrayList<Literal>(terms));
		}
		this.negated = negated;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		Predicate p = (Predicate) super.clone();
		p.setNegated(negated);
		return p;
	}

	public boolean isNegated() {
		return negated;
	}

	public void setNegated(boolean negated) {
		this.negated = negated;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();

		if (precedingComments != null && precedingComments.size() > 0) {
			for (BlockComment bc : precedingComments) {
				sb.append(" ");
				sb.append(bc.getText());
				sb.append(" ");
			}
		}

		if (signed) {
			sb.append("not ");
		}

		if (negated) {
			sb.append("-");
		}

		if (identifier != null)
			sb.append(identifier);

		if (terms != null && terms.size() > 0) {
			sb.append("(");

			for (int i = 0; i < terms.size() - 1; i++) {
				sb.append(terms.get(i).toString());
				sb.append(",");
			}

			sb.append(terms.get(terms.size() - 1).toString());
			sb.append(")");
		}

		if (succeedingComments != null && succeedingComments.size() > 0) {
			for (BlockComment bc : succeedingComments) {
				sb.append(" ");
				sb.append(bc.getText());
				sb.append(" ");
			}
		}
		return sb.toString();

	}

}
