/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class RelLit extends Literal {

	public static final String EQUAL = "==";
	public static final String LTHAN = "<=";
	public static final String GTHAN = ">=";
	public static final String LOWER = "<";
	public static final String GREATER = ">";
	public static final String INEQUAL = "!=";
	public static final String ASSIGN = "=";
	public static final String CASSIGN = ":=";

	protected RelLit() {
		super();
	}

	public RelLit(String c, Term a, AggregateLiteral b, int lineNo) {
		super(c, null, false, lineNo);
		List<Literal> termList = new ArrayList<Literal>();
		termList.add(a);
		termList.add(b);
		this.setTerms(termList);
	}

	public RelLit(String c, Term a, Term b) {
		super(c, null, false);
		List<Literal> termList = new ArrayList<Literal>();
		termList.add(a);
		termList.add(b);
		this.setTerms(termList);
	}

	public RelLit(String c, Term a, Term b, int lineNo) {
		super(c, null, false, lineNo);
		List<Literal> termList = new ArrayList<Literal>();
		termList.add(a);
		termList.add(b);
		this.setTerms(termList);
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (precedingComments != null && precedingComments.size() > 0) {
			for (BlockComment bc : precedingComments) {
				sb.append(bc.getText());
			}

			sb.append(" ");
		}
		sb.append(terms.get(0).toString() + " " + this.getIdentifier() + " "
				+ terms.get(1).toString());

		if (succeedingComments != null && succeedingComments.size() > 0) {
			for (BlockComment bc : succeedingComments) {
				sb.append(bc.getText());
			}
		}
		return sb.toString();
	}

}
