/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

public class VarTerm extends Term {

	public VarTerm() {
		super(null);
	}

	public VarTerm(String varName) {
		super(varName);
	}

	public VarTerm(String varName, int lineNo) {
		super(varName, lineNo);
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		VarTerm newTerm;

		if (identifier != null)
			newTerm = new VarTerm(new String(identifier), lineNo);
		else
			newTerm = new VarTerm(null, lineNo);
		List<BlockComment> newPrecedingComments = null;

		if (precedingComments != null) {
			newPrecedingComments = new ArrayList<BlockComment>();
			for (BlockComment c : precedingComments) {
				newPrecedingComments.add((BlockComment) c.clone());
			}
		}

		List<BlockComment> newSucceedingComments = null;
		if (succeedingComments != null) {
			newSucceedingComments = new ArrayList<BlockComment>();
			for (BlockComment c : succeedingComments) {
				newSucceedingComments.add((BlockComment) c.clone());
			}
		}

		newTerm.setPrecedingComments(newPrecedingComments);
		newTerm.setSucceedingComments(newSucceedingComments);

		return newTerm;
	}

	@Override
	public List<VarTerm> getVariables() {
		List<VarTerm> tmp = new ArrayList<VarTerm>();
		tmp.add(this);
		return tmp;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (precedingComments != null && precedingComments.size() > 0) {
			for (BlockComment bc : precedingComments) {
				sb.append(bc.getText());
			}
		}
		if (identifier != null)
			sb.append(identifier);
		else
			sb.append("_");

		if (succeedingComments != null && succeedingComments.size() > 0) {
			for (BlockComment bc : succeedingComments) {
				sb.append(bc.getText());
			}
		}
		return sb.toString();
	}

}
