/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package main;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import aspmodificator.modificationrules.IModificationRule;
import aspmodificator.modificationrules.gringo.AddDefaultNegationModificationRule;
import aspmodificator.modificationrules.gringo.AddStrongNegationModificationRule;
import aspmodificator.modificationrules.gringo.AggregateLiteralModificationRule;
import aspmodificator.modificationrules.gringo.ConstantToVariableModificationRule;
import aspmodificator.modificationrules.gringo.DeleteRuleModificationRule;
import aspmodificator.modificationrules.gringo.DeleteTermModificationRule;
import aspmodificator.modificationrules.gringo.LogicRuleTypeEnum;
import aspmodificator.modificationrules.gringo.MathOperatorsModificationRule;
import aspmodificator.modificationrules.gringo.RangeTermModificationRule;
import aspmodificator.modificationrules.gringo.RelationOperatorModificationRule;
import aspmodificator.modificationrules.gringo.RemoveDefaultNegationModificationRule;
import aspmodificator.modificationrules.gringo.RemoveStrongNegationModificationRule;
import aspmodificator.modificationrules.gringo.RenamePredicateModificationRule;
import aspmodificator.modificationrules.gringo.RenameVariableModificationRule;
import aspmodificator.modificationrules.gringo.ReplaceConstantModificationRule;
import aspmodificator.modificationrules.gringo.ReplaceLiteralModificationRule;
import aspmodificator.modificationrules.gringo.SwapStrongNegationModificationRule;
import aspmodificator.modificationrules.gringo.SwapTermModificationRule;
import aspmodificator.modificationrules.gringo.SwitchDefaultNegationModificationRule;
import aspmodificator.modificationrules.gringo.RuleTargetEnum;
import aspmodificator.modificationrules.gringo.TwiddleAggregateWeightsModificationRule;
import aspmodificator.modificationrules.gringo.TwiddleMathStatementModificationRule;
import aspmodificator.modificationrules.gringo.VariableToConstantModificationRule;

public class AspModificationConfig {
	private static final String CONFIGURATIONTAG = "configuration";
	private static final String MODIFICATION_RULES_TAG = "modificationRules";
	private static final String MODIFICATION_RULE_TAG = "modificationRule";
	private static final String ATTRIBUTE_TYPE = "type";
	private static final String ATTRIBUTE_MIN = "min";
	private static final String ATTRIBUTE_MAX = "max";
	private static final String MR_DELETERULE_TAG = "deleteRule";
	private static final String MR_DELETELITERAL_TAG = "deleteLiteral";
	private static final String MR_RANGETERM_TAG = "changeRangeTerms";
	private static final String MR_RELATIONOPERATOR_TAG = "changeRelationOperators";
	private static final String MR_SWAPTERM_TAG = "swapTerms";
	private static final String MR_AGGREGATE_TAG = "changeAggregates";
	private static final String MR_MATHOPERATORS_TAG = "changeMathOperators";
	private static final String MR_SWAP_DEFAULT_NEGATION_TAG = "swapDefaultNegation";
	private static final String MR_SWAP_STRONG_NEGATION_TAG = "swapStrongNegation";
	private static final String ATTRIBUTE_SWAP_STRONG_NEGATION_REQUIREEXISTINGPREDICATE = "requireExistingObject";
	private static final String ATTRIBUTE_VARIABLE_TO_CONSTANT_REQUIREEXISTINGPREDICATE = "requireExistingObject";
	private static final String MR_VARIABLE_TO_CONSTANT_TAG = "changeVariableToConstant";
	private static final String MR_CONSTANT_TO_VARIABLE_TAG = "changeConstantToVariable";
	private static final String MR_REPLACE_CONSTANTS = "replaceConstants";
	private static final String MR_REPLACE_LITERALS = "replaceGroundedLiterals";
	private static final String MR_TWIDDLEMATHSTATEMENT = "twiddleMathStatement";
	private static final String MR_TWIDDLEAGGREGATEWEIGHT = "twiddleAggregateWeight";
	private static final String MR_RENAME_PREDICATE_TAG = "renamePredicates";
	private static final String MR_ADD_DEFAULT_NEGATION_TAG = "addDefaultNegation";
	private static final String MR_REMOVE_DEFAULT_NEGATION_TAG = "removeDefaultNegation";
	private static final String MR_ADD_STRONG_NEGATION_TAG = "addStrongNegation";
	private static final String MR_REMOVE_STRONG_NEGATION_TAG = "removeStrongNegation";
	private static final String CF_RULE_APPLICATION = "ruleApplication";
	private static final String CF_WRITE_COMMENTS = "writeComments";
	private static final String CF_OUTPUT_DIRECTORY = "outputDirectory";
	private static final String CF_INPUT_FILE = "inputFile";
	private static final String CF_RANDOM_INIT = "randomInit";
	private static final String CF_NO_COPIES = "noCopies";
	private static final String CF_USE_MEMORY = "useMemory";
	private static final String ATTRIBUTE_DELETELITERAL_INCLUDEINSIDELITERALS = "includeInsideLiterals";
	private static final String ATTRIBUTE_CHANGERELATIONOPERATOR_AVOIDNEARLYINEQUALOPERATORS = "avoidNearlyInequalOperators";
	private static final String ATTRIBUTE_TWIDDLEMATHSTATEMENT_INCLUDERELATIONS = "includeRelations";
	private static final String ATTRIBUTE_TWIDDLEMATHSTATEMENT_INCLUDEMATHTERMS = "includeMathTerms";
	private static final String ATTRIBUTE_MR_RULETARGET = "targetLiterals";
	private static final String MR_RDP_TAG = "RDP"; // delete proper rule
	private static final String MR_RDC_TAG = "RDC"; // delete constraint
	private static final String MR_RDF_TAG = "RDF"; // delete fact
	private static final String MR_LDB_TAG = "LDB"; // delete body literal
	private static final String MR_LDH_TAG = "LDH"; // delete head literal
	private static final String MR_LAD_TAG = "LAD"; // add default negation
	private static final String MR_LRD_TAG = "LRD"; // remove default negation
	private static final String MR_LAS_TAG = "LAS"; // add strong negation
	private static final String MR_LRS_TAG = "LRS"; // remove strong negation
	private static final String MR_LRP_TAG = "LRP"; // rename predicate
	private static final String MR_LRC_TAG = "LRC"; // replace comparison
													// relation
	private static final String MR_ARO_TAG = "ARO"; // replace arithmetic
													// operator
	private static final String MR_ATV_TAG = "ATV"; // twiddle variable domain
	private static final String MR_ATA_TAG = "ATA"; // twiddle aggregate bound
	private static final String MR_ATW_TAG = "ATW"; // twiddle aggregate weight
	private static final String MR_TST_TAG = "TST"; // swap terms in literals
	private static final String MR_TVC_TAG = "TVC"; // change variable to
													// constant
	private static final String MR_TRV_TAG = "TRV"; // rename variable
	private static final String MR_TCV_TAG = "TCV"; // change constant to
													// variable
	private static final String MR_TRC_TAG = "TRC"; // rename constant

	public static AspModificationConfig createConfigFromFile(File file)
			throws Exception {

		if (!file.exists())
			throw new FileNotFoundException("Config-file not found");

		// Erzeugen eines JDOM-Dokuments anhand der Datei
		SAXBuilder builder = new SAXBuilder();
		Document doc = builder.build(file);

		return createAspModificationConfigFromDoc(doc);

	}

	@SuppressWarnings("unchecked")
	private static void doCommonConfiguration(Element rootElement,
			AspModificationConfig config) {
		List<Element> children = rootElement.getChildren();
		for (Element el : children) {
			try {
				handleConfigElement(el, config);
			} catch (Exception e) {
				logger.error("Error while reading config-file", e);
			}
		}
	}

	private static IModificationRule generateRule(Element element)
			throws Exception {
		String type = element.getAttributeValue(ATTRIBUTE_TYPE);

		if (type == null || type == "")
			return null;
		int minApplication = 1;
		int maxApplication = 1;

		String min = element.getAttributeValue(ATTRIBUTE_MIN);
		if (min != null && min != "") {
			minApplication = Integer.parseInt(min);
			if (minApplication < 0)
				throw new Exception("min-value must be greater or equal 0");
		}

		String max = element.getAttributeValue(ATTRIBUTE_MAX);
		if (max != null && max != "") {
			maxApplication = Integer.parseInt(max);
			if (maxApplication < 0)
				throw new Exception("max-value must be greater or equal 0");

			if (maxApplication < minApplication)
				throw new Exception("max-value must be less or equal min-value");
		}

		return generateRuleByType(type, minApplication, maxApplication, element);
	}

	private static IModificationRule generateRuleByType(String type,
			int minApplication, int maxApplication, Element element) {

		// rule target
		String targetLiteralString = element
				.getAttributeValue(ATTRIBUTE_MR_RULETARGET);
		RuleTargetEnum targetEnum = null;
		if (targetLiteralString != null && targetLiteralString != "") {
			targetEnum = RuleTargetEnum.valueOf(targetLiteralString);
		}

		// different rule types
		if (type.equals(MR_DELETERULE_TAG)) {
			return new DeleteRuleModificationRule(minApplication,
					maxApplication);
		}
		if (type.equals(MR_DELETELITERAL_TAG)) {
			String includeInsideLiteralsString = element
					.getAttributeValue(ATTRIBUTE_DELETELITERAL_INCLUDEINSIDELITERALS);
			boolean includeInsideLiterals = false;

			if (includeInsideLiteralsString != null
					&& includeInsideLiteralsString != "")
				includeInsideLiterals = Boolean
						.parseBoolean(includeInsideLiteralsString);

			DeleteTermModificationRule tmpRule = new DeleteTermModificationRule(
					minApplication, maxApplication);
			tmpRule.setIncludeInsideLiterals(includeInsideLiterals);
			if (targetEnum != null)
				tmpRule.setMrRuleTarget(targetEnum);
			return tmpRule;
		}
		if (type.equals(MR_RANGETERM_TAG)) {
			RangeTermModificationRule rule = new RangeTermModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}
		if (type.equals(MR_RELATIONOPERATOR_TAG)) {
			String avoidNearlyInequalOperatorsString = element
					.getAttributeValue(ATTRIBUTE_CHANGERELATIONOPERATOR_AVOIDNEARLYINEQUALOPERATORS);
			boolean avoidNearlyInequalOperators = false;

			if (avoidNearlyInequalOperatorsString != null
					&& avoidNearlyInequalOperatorsString != "")
				avoidNearlyInequalOperators = Boolean
						.parseBoolean(avoidNearlyInequalOperatorsString);

			RelationOperatorModificationRule rule = new RelationOperatorModificationRule(
					minApplication, maxApplication, avoidNearlyInequalOperators);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}
		if (type.equals(MR_SWAPTERM_TAG)) {
			SwapTermModificationRule rule = new SwapTermModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_AGGREGATE_TAG)) {
			AggregateLiteralModificationRule rule = new AggregateLiteralModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}
		if (type.equals(MR_MATHOPERATORS_TAG)) {
			return new MathOperatorsModificationRule(minApplication,
					maxApplication);
		}

		if (type.equals(MR_SWAP_DEFAULT_NEGATION_TAG)) {
			return new SwitchDefaultNegationModificationRule(minApplication,
					maxApplication);
		}
		if (type.equals(MR_SWAP_STRONG_NEGATION_TAG)) {
			String requireExistingPredicateString = element
					.getAttributeValue(ATTRIBUTE_SWAP_STRONG_NEGATION_REQUIREEXISTINGPREDICATE);
			boolean requireExistingPredicate = false;

			if (requireExistingPredicateString != null
					&& requireExistingPredicateString != "")
				requireExistingPredicate = Boolean
						.parseBoolean(requireExistingPredicateString);

			SwapStrongNegationModificationRule rule = new SwapStrongNegationModificationRule(
					minApplication, maxApplication, requireExistingPredicate);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_VARIABLE_TO_CONSTANT_TAG)) {

			String requireExistingConstantString = element
					.getAttributeValue(ATTRIBUTE_VARIABLE_TO_CONSTANT_REQUIREEXISTINGPREDICATE);
			RuleTargetEnum requireExistingEnum = RuleTargetEnum.NONE;

			if (requireExistingConstantString != null
					&& requireExistingConstantString != "")
				requireExistingEnum = RuleTargetEnum
						.valueOf(requireExistingConstantString);

			VariableToConstantModificationRule rule = new VariableToConstantModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			rule.setExistenceCheckTarget(requireExistingEnum);
			return rule;
		}

		if (type.equals(MR_CONSTANT_TO_VARIABLE_TAG)) {
			ConstantToVariableModificationRule rule = new ConstantToVariableModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_RENAME_PREDICATE_TAG)) {
			RenamePredicateModificationRule rule = new RenamePredicateModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_REPLACE_CONSTANTS)) {
			ReplaceConstantModificationRule rule = new ReplaceConstantModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_REPLACE_LITERALS)) {
			ReplaceLiteralModificationRule rule = new ReplaceLiteralModificationRule(
					minApplication, maxApplication);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_ADD_DEFAULT_NEGATION_TAG)) {
			return new AddDefaultNegationModificationRule(minApplication,
					maxApplication);
		}

		if (type.equals(MR_REMOVE_DEFAULT_NEGATION_TAG)) {
			return new RemoveDefaultNegationModificationRule(minApplication,
					maxApplication);
		}

		if (type.equals(MR_ADD_STRONG_NEGATION_TAG)) {
			String requireExistingPredicateString = element
					.getAttributeValue(ATTRIBUTE_SWAP_STRONG_NEGATION_REQUIREEXISTINGPREDICATE);
			boolean requireExistingPredicate = false;

			if (requireExistingPredicateString != null
					&& requireExistingPredicateString != "")
				requireExistingPredicate = Boolean
						.parseBoolean(requireExistingPredicateString);

			AddStrongNegationModificationRule rule = new AddStrongNegationModificationRule(
					minApplication, maxApplication, requireExistingPredicate);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_REMOVE_STRONG_NEGATION_TAG)) {
			String requireExistingPredicateString = element
					.getAttributeValue(ATTRIBUTE_SWAP_STRONG_NEGATION_REQUIREEXISTINGPREDICATE);
			boolean requireExistingPredicate = false;

			if (requireExistingPredicateString != null
					&& requireExistingPredicateString != "")
				requireExistingPredicate = Boolean
						.parseBoolean(requireExistingPredicateString);

			RemoveStrongNegationModificationRule rule = new RemoveStrongNegationModificationRule(
					minApplication, maxApplication, requireExistingPredicate);
			if (targetEnum != null)
				rule.setMrRuleTarget(targetEnum);
			return rule;
		}

		if (type.equals(MR_TWIDDLEMATHSTATEMENT)) {

			String includeRelationsString = element
					.getAttributeValue(ATTRIBUTE_TWIDDLEMATHSTATEMENT_INCLUDERELATIONS);
			boolean includeRelations = true;

			if (includeRelationsString != null && includeRelationsString != "")
				includeRelations = Boolean.parseBoolean(includeRelationsString);

			String includeMathTermsString = element
					.getAttributeValue(ATTRIBUTE_TWIDDLEMATHSTATEMENT_INCLUDEMATHTERMS);
			boolean includeMathTerms = true;

			if (includeMathTermsString != null && includeMathTermsString != "")
				includeMathTerms = Boolean.parseBoolean(includeMathTermsString);

			return new TwiddleMathStatementModificationRule(minApplication,
					maxApplication, includeRelations, includeMathTerms);
		}

		if (type.equals(MR_TWIDDLEAGGREGATEWEIGHT)) {
			TwiddleAggregateWeightsModificationRule rule = new TwiddleAggregateWeightsModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		// shortcuts
		if (type.equals(MR_RDP_TAG)) {
			DeleteRuleModificationRule rule = new DeleteRuleModificationRule(
					minApplication, maxApplication);
			rule.setRuleType(LogicRuleTypeEnum.PROPER);
			return rule;
		}

		if (type.equals(MR_RDC_TAG)) {
			DeleteRuleModificationRule rule = new DeleteRuleModificationRule(
					minApplication, maxApplication);
			rule.setRuleType(LogicRuleTypeEnum.CONSTRAINT);
			return rule;
		}

		if (type.equals(MR_RDF_TAG)) {
			DeleteRuleModificationRule rule = new DeleteRuleModificationRule(
					minApplication, maxApplication);
			rule.setRuleType(LogicRuleTypeEnum.FACT);
			return rule;
		}

		if (type.equals(MR_LDB_TAG)) {
			DeleteTermModificationRule rule = new DeleteTermModificationRule(
					minApplication, maxApplication);
			rule.setMrRuleTarget(RuleTargetEnum.BODY);
			return rule;
		}

		if (type.equals(MR_LDH_TAG)) {
			DeleteTermModificationRule rule = new DeleteTermModificationRule(
					minApplication, maxApplication);
			rule.setMrRuleTarget(RuleTargetEnum.HEAD);
			return rule;
		}

		if (type.equals(MR_LAD_TAG)) {
			AddDefaultNegationModificationRule rule = new AddDefaultNegationModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_LRD_TAG)) {
			RemoveStrongNegationModificationRule rule = new RemoveStrongNegationModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_LAS_TAG)) {
			AddStrongNegationModificationRule rule = new AddStrongNegationModificationRule(
					minApplication, maxApplication);
			rule.setRequireExistingObject(true);
			return rule;
		}

		if (type.equals(MR_LRS_TAG)) {
			RemoveStrongNegationModificationRule rule = new RemoveStrongNegationModificationRule(
					minApplication, maxApplication);
			rule.setRequireExistingObject(true);
			return rule;
		}

		if (type.equals(MR_LRP_TAG)) {
			RenamePredicateModificationRule rule = new RenamePredicateModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_LRC_TAG)) {
			ReplaceConstantModificationRule rule = new ReplaceConstantModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_ARO_TAG)) {
			MathOperatorsModificationRule rule = new MathOperatorsModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_ATV_TAG)) {
			TwiddleMathStatementModificationRule rule = new TwiddleMathStatementModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_ATA_TAG)) {
			AggregateLiteralModificationRule rule = new AggregateLiteralModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_ATW_TAG)) {
			TwiddleAggregateWeightsModificationRule rule = new TwiddleAggregateWeightsModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_TST_TAG)) {
			SwapTermModificationRule rule = new SwapTermModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_TVC_TAG)) {
			VariableToConstantModificationRule rule = new VariableToConstantModificationRule(
					minApplication, maxApplication);
			rule.setExistenceCheckTarget(RuleTargetEnum.ALL);
			return rule;
		}

		if (type.equals(MR_TRV_TAG)) {
			RenameVariableModificationRule rule = new RenameVariableModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_TCV_TAG)) {
			ConstantToVariableModificationRule rule = new ConstantToVariableModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		if (type.equals(MR_TRC_TAG)) {
			ReplaceConstantModificationRule rule = new ReplaceConstantModificationRule(
					minApplication, maxApplication);
			return rule;
		}

		logger.warn("Unknown rule-tag: " + type);
		return null;
	}

	private static void handleConfigElement(Element el,
			AspModificationConfig config) {
		String name = el.getName();
		if (name.equals(CF_RULE_APPLICATION)) {
			String value = el.getAttributeValue("application");
			if (value != null)
				config.setRuleApplication(RuleApplicationEnum.valueOf(value));
			else
				logger.warn(CF_RULE_APPLICATION + " tag without value found");
			return;
		}

		if (name.equals(CF_WRITE_COMMENTS)) {
			String value = el.getAttributeValue("value");
			if (value != null)
				config.setWriteComments(Boolean.parseBoolean(value));
			else
				logger.warn(CF_WRITE_COMMENTS + " tag without value found");
			return;
		}

		if (name.equals(CF_OUTPUT_DIRECTORY)) {
			String value = el.getAttributeValue("value");

			if (value == null)
				logger.warn(CF_OUTPUT_DIRECTORY + " tag without value found");

			File tmpFile = new File(value);

			if (!tmpFile.isDirectory()) {
				logger.warn("Outputpath in logfile is not an existing directory");
				return;
			}

			config.setOutputDir(tmpFile);
			return;
		}
		if (name.equals(CF_INPUT_FILE)) {
			String value = el.getAttributeValue("value");

			if (value == null || value == "")
				logger.warn(CF_INPUT_FILE + " tag without value found");

			File tmpFile = new File(value);
			if (!tmpFile.exists() || tmpFile.isDirectory()) {
				System.out.println(value + " does not exist or is a directory");

			}
			config.setInputFile(tmpFile);
			return;
		}

		if (name.equals(CF_RANDOM_INIT)) {
			String value = el.getAttributeValue("value");

			if (value == null || value == "")
				logger.warn(CF_RANDOM_INIT + " tag without value found");

			config.setRandomInit(Long.parseLong(value));
			return;
		}
		if (name.equals(CF_NO_COPIES)) {
			String value = el.getAttributeValue("value");

			if (value == null || value == "")
				logger.warn(CF_NO_COPIES + " tag without value found");

			config.setNoCopies(Integer.parseInt(value));
			return;
		}

		if (name.equals(CF_USE_MEMORY)) {
			String value = el.getAttributeValue("value");
			if (value != null)
				config.setUseMemory(Boolean.parseBoolean(value));
			else
				logger.warn(CF_USE_MEMORY + " tag without value found");
			return;
		}

		// unknown config-element
		logger.warn("Unknown config-element: " + name);
	}

	@SuppressWarnings("unchecked")
	private static void setModificationRules(Element modRules,
			AspModificationConfig config) throws Exception {

		List<Element> rules = modRules.getChildren(MODIFICATION_RULE_TAG);

		if (rules == null || rules.size() == 0)
			throw new Exception("No modification rules found in config-file");

		List<IModificationRule> ruleList = new ArrayList<IModificationRule>();

		for (Element element : rules) {
			IModificationRule rule = generateRule(element);
			if (rule != null)
				ruleList.add(rule);
		}
		config.setModificationRules(ruleList);
	}

	protected File outputDir = null;

	protected File inputFile;

	protected int noCopies = 1;

	protected boolean writeComments = false;

	protected String outputFileName = null;

	protected boolean useMemory = true;

	private static Logger logger = Logger
			.getLogger(AspModificationConfig.class);

	protected static AspModificationConfig createAspModificationConfigFromDoc(
			Document doc) throws Exception {
		AspModificationConfig config = new AspModificationConfig();

		// Lesen des Wurzelelements des JDOM-Dokuments doc
		Element root = doc.getRootElement();

		Element configuration = root.getChild(CONFIGURATIONTAG);

		if (configuration != null) {
			doCommonConfiguration(configuration, config);
		} else {
			// use default configuration
		}

		Element modRules = root.getChild(MODIFICATION_RULES_TAG);

		if (modRules == null)
			throw new Exception("No element with <" + MODIFICATION_RULES_TAG
					+ "> found!");

		setModificationRules(modRules, config);

		return config;
	}

	public static AspModificationConfig createConfigFromFile(String filePath)
			throws Exception {

		if (filePath == null)
			throw new NullPointerException(
					"Path to config-file must not be null");

		File file = new File(filePath);
		if (!file.exists())
			throw new FileNotFoundException("Config-file not found");

		// Erzeugen eines JDOM-Dokuments anhand der Datei
		SAXBuilder builder = new SAXBuilder();
		Document doc = builder.build(file);

		return createAspModificationConfigFromDoc(doc);

	}

	private RuleApplicationEnum ruleApplication = RuleApplicationEnum.all;

	private List<IModificationRule> modificationRules;

	private long randomInit = System.currentTimeMillis();

	public File getInputFile() {
		return inputFile;
	}

	public List<IModificationRule> getModificationRules() {
		return modificationRules;
	}

	public int getNoCopies() {
		return noCopies;
	}

	public File getOutputDir() {
		return outputDir;
	}

	public String getOutputFileName() {
		return outputFileName;
	}

	public long getRandomInit() {
		return randomInit;
	}

	public RuleApplicationEnum getRuleApplication() {
		return ruleApplication;
	}

	public boolean isUseMemory() {
		return useMemory;
	}

	public boolean isWriteComments() {
		return writeComments;
	}

	public void setInputFile(File inputFile) {
		this.inputFile = inputFile;
	}

	public void setModificationRules(List<IModificationRule> modificationRules) {
		this.modificationRules = modificationRules;
	}

	public void setNoCopies(int noCopies) {
		this.noCopies = noCopies;
	}

	public void setOutputDir(File outputDir) {
		this.outputDir = outputDir;
	}

	public void setOutputFileName(String outputFileName) {
		this.outputFileName = outputFileName;
	}

	public void setRandomInit(long randomInit) {
		this.randomInit = randomInit;
	}

	private void setRuleApplication(RuleApplicationEnum ruleApplication) {
		this.ruleApplication = ruleApplication;
	}

	public void setUseMemory(boolean useMemory) {
		this.useMemory = useMemory;
	}

	public void setWriteComments(boolean writeComments) {
		this.writeComments = writeComments;
	}

}
