/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package main;

import java.io.File;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

public class Main {

	/* private static final Object DIALECT = "-dialect"; */

	private static Logger logger = Logger.getLogger(Main.class);

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		BasicConfigurator.configure();

		Main main = new Main();
		CmdLineParser parser = new CmdLineParser(main);
		parser.setUsageWidth(80); // width of the error display area

		try {
			parser.parseArgument(args);
		} catch (CmdLineException e) {
			System.err.println(e.getMessage());
			// System.err
			// .println("java -jar <programname>.jar [options...] arguments...");
			// print the list of available options
			parser.printUsage(System.err);
			System.err.println();
			System.exit(1);
		}

		try {
			main.run();
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	@Option(name = "-r", aliases = "-randomInit", usage = "initialization-seed for the random-number generator")
	private Long randomInitValue = null;

	@Option(name = "-noComments", usage = "initialization-seed for the random-number generator")
	private Boolean writeComments = true;

	@Option(name = "-o", aliases = { "-out", "-outputDir" }, usage = "the directory where the modified programs should be saved")
	private File outputDir = null;

	@Option(name = "-c", aliases = "-configFile", required = true, usage = "path to the config file")
	private File configFile = null;

	@Option(name = "-i", aliases = { "-input", "-inputFile" }, usage = "file containing an asp-program which should be the base program for the modification")
	private File inputFile = null;

	@Option(name = "-noCopies", usage = "number of modified output-programs")
	private Integer noCopies = null;

	@Option(name = "-logging", aliases = "-log", usage = "logging options - one of the following logging-levels: "
			+ "\"all\", \"debug\", \"error\", \"fatal\", \"info\", \"off\", \"trace\", \"warn")
	private String loggingLevel = Level.INFO.toString();

	public void run() throws Exception {

		logger.setLevel(Level.toLevel(loggingLevel));

		AspModificationConfig config = null;
		config = AspModificationConfig.createConfigFromFile(configFile);

		AspModificator aspModificator = new AspModificator();

		aspModificator.setConfig(config);

		// override all config-params passed via arguments
		if (randomInitValue != null)
			aspModificator.setRandomInitialization(randomInitValue);

		if (writeComments != null)
			aspModificator.setWriteComments(writeComments);

		if (noCopies != null)
			aspModificator.setNoCopies(noCopies);

		if (outputDir != null)
			aspModificator.setOutputDir(outputDir);
		if (inputFile != null)
			aspModificator.setInputFile(inputFile);

		if (outputDir != null && !outputDir.isDirectory()) {
			System.out.println("output path must be an existing directory");
			System.exit(1);
		}

		if (inputFile != null
				&& (!inputFile.exists() || inputFile.isDirectory())) {
			System.out.println(inputFile.getAbsolutePath()
					+ " does not exist or is a directory");
			System.exit(1);
		}

		try {
			aspModificator.doYourJob();
		} catch (Exception e) {
			logger.error("Error while working", e);
			e.printStackTrace();
		}
	}
}
