/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;

public class RemoveDefaultNegationModificationRuleTest {

	private GringoParser parser;
	private RemoveDefaultNegationModificationRule rule;

	@Before
	public void setUp() throws Exception {
		parser = new GringoParser();
		rule = new RemoveDefaultNegationModificationRule();
	}

	@Test
	public void testApply_shouldRemoveDefaultNegation_1() throws Exception {
		String code = "p :- q(X,Y), not r(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMrRuleTarget(RuleTargetEnum.ALL);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertThat(res, is(1));
		assertThat(program.get(0).toString(), is("p :- q(X,Y), r(X)."));
	}
}
