/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;

public class GringoRuleTest {
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		parser = new GringoParser();
	}

	@Test
	public void testToOutputString_1() throws Exception {
		String text = ":- move(T,D,X,Y), stone(X,Y,T), dir(D,DX,DY).";

		List<LineBase> result = parser.parse(text);
		GringoRule rule = (GringoRule) result.get(0);

		assertEquals(text, rule.toOutputString());
	}

	@Test
	public void testToOutputString_2() throws Exception {
		String text = "move(T,D,X,Y) :- stone(X,Y,T), dir(D,DX,DY).";

		List<LineBase> result = parser.parse(text);
		GringoRule rule = (GringoRule) result.get(0);

		assertEquals(text, rule.toOutputString());
	}

	@Test
	public void testToOutputString_3() throws Exception {
		String text = "move(T,D,X,Y) :- \nstone(X,Y,T), dir(D,DX,DY).";

		List<LineBase> result = parser.parse(text);
		GringoRule rule = (GringoRule) result.get(0);

		assertEquals(text, rule.toOutputString());
	}

	@Test
	public void testToOutputString_4() throws Exception {
		String text = "move(T,D,X,Y) | -move(T,D,X,Y) :- \nstone(X,Y,T), \ndir(D,DX,DY).";

		List<LineBase> result = parser.parse(text);
		GringoRule rule = (GringoRule) result.get(0);

		assertEquals(text, rule.toOutputString());
	}

	@Test
	public void testToOutputString_5() throws Exception {
		String text = "move(T,D,X,Y) | \n-move(T,D,X,Y) :- \nstone(X,Y,T), \ndir(D,DX,DY).";

		List<LineBase> result = parser.parse(text);
		GringoRule rule = (GringoRule) result.get(0);

		assertEquals(text, rule.toOutputString());
	}
}
